#
#   Yasuharu Okamoto, 2020.08
#
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import pystan
import seaborn as sb

finnm = input('入力データファイル（*.xlsx）= ')
txtoutnm = input('出力ファイル（*.txt）= ')
foutnm = input('画像保存ファイル（*.png）= ')
txtout = open(txtoutnm, 'w')
#
#      Excelファイルの読み込み
#　　　モジュール xlrd がインストール済みであること
#
xlsx_fl = pd.ExcelFile(finnm)                       #   ExcelファイルからExcelFile型オブジェクトを生成
data_xlsx = pd.read_excel(xlsx_fl,  header = None)  #   ExcelFile型オブジェクトからDataFrame型オブジェクトを生成
data = data_xlsx.values                             #   Dataframe型オブジェクトからnumpyの配列オブジェクトを生成
print('\n', data)

txtout.write('\n入力データファイル: {}\n'.format(finnm))

NameA = data[0][0].split(sep = '/')[0]
a = int(data[0][0].split(sep = '/')[1])
NameB = data[1][0].split(sep = '/')[0]
b = int(data[1][0].split(sep = '/')[1])

LevelsA = [data[0][1 + j*b] for j in range(a)]
LevelsB = data[1][1:b+1]

print('\nNameA = ', NameA)
print('a = ', a)
print('LevelsA: ', LevelsA)
print('NameB = ', NameB)
print('b = ', b)
print('LevelB: ', LevelsB)

txtout.write('\nNameA = {}\n'.format(NameA))
txtout.write('a = {}\n'.format(a))
txtout.write('LevelsA: {}\n'.format(LevelsA))
txtout.write('\nNameB = {}\n'.format(NameB))
txtout.write('b = {}\n'.format(b))
txtout.write('LevelB: {}\n'.format(LevelsB))

N = len(data) - 2
print('N = ', N)
txtout.write('\nN = {}\n'.format(N))

X = np.empty((N, a, b))
for i in range(N):
    for j in range(a):
        for k in range(b):
            X[i][j][k] = data[i + 2][1 + j * b + k]
print('\nX =')
txtout.write('\nX =\n')
for i in range(N):
    for j in range(a):
        for k in range(b):
            print('  ', X[i][j][k], end = '')
            txtout.write('  {}'.format(X[i][j][k]))
    print()
    txtout.write('\n')
  
StanData = {'a': a, 'b': b, 'N': N, 'X': X}

sm = pystan.StanModel(file = 'TwoFctrFixed.stan')
fit = sm.sampling(data = StanData, n_jobs = 1)
print(fit)

txtout.write('\n{}\n'.format(fit))

N_Samples = len(fit['G'])
#
#       主効果（f_a, f_b）と交互作用（f_ab）
#
f_as = np.empty((N_Samples, a))
f_bs = np.empty((N_Samples, b))
f_abs = np.empty((N_Samples, a, b))
n_s = 0
for vG in fit['G']:
    Gdd = vG.mean()             #   G..
    Gjd = vG.mean(axis = 1)     #   Gj.
    Gdk = vG.mean(axis = 0)     #   G.k
   
    f_a = Gjd - Gdd
    f_b = Gdk - Gdd
    f_ab = vG - Gjd.reshape((a,1)) - Gdk + Gdd
    f_as[n_s] = f_a
    f_bs[n_s] = f_b
    f_abs[n_s] = f_ab
    n_s += 1

#       日本語表記の設定
plt.rcParams['font.family'] = ['Yu Gothic', 'Noto Sans CJK JP', 'Hiragino Maru Gothic Pro']

print('\n主効果：{}'.format(NameA))
txtout.write('\n主効果：{}\n'.format(NameA))
for j in range(a):
    print('\n{}：'.format(LevelsA[j]))
    txtout.write('\n{}：\n'.format(LevelsA[j]))
    v_mean = f_as[:, j].mean()
    v_025p, v_25p, v_med, v_75p, v_975p = \
        np.percentile(f_as[:, j], [2.5, 25, 50, 75, 97.5])
    print('    平均値 = {0:.3f}'.format(v_mean))
    print('    中央値 = {0:.3f}'.format(v_med))
    print('    Q1 = {0:.3f}    Q3 = {1:.3f}'.format(v_25p, v_75p))
    print('    95% CI = [{0:.3f}, {1:.3f}]'.format(v_025p, v_975p))
    txtout.write('    平均値 = {0:.3f}\n'.format(v_mean))
    txtout.write('    中央値 = {0:.3f}\n'.format(v_med))
    txtout.write('    Q1 = {0:.3f}    Q3 = {1:.3f}\n'.format(v_25p, v_75p))
    txtout.write('    95% CI = [{0:.3f}, {1:.3f}]\n'.format(v_025p, v_975p))

plt.title('主効果：{}'.format(NameA), fontsize = 20)
for j in range(a):
    sb.kdeplot(f_as[:, j], label = LevelsA[j])
plt.savefig(foutnm[:-4] + '_a.png')
plt.yticks([])
SavedFiles = [foutnm[:-4] + '_a.png']
plt.show()

print('\n主効果：{}'.format(NameB))
txtout.write('\n主効果：{}\n'.format(NameB))
for k in range(b):
    print('\n{}：'.format(LevelsB[k]))
    txtout.write('\n{}：\n'.format(LevelsB[k]))
    v_mean = f_bs[:, k].mean()
    v_025p, v_25p, v_med, v_75p, v_975p = \
        np.percentile(f_bs[:, k], [2.5, 25, 50, 75, 97.5])
    print('    平均値 = {0:.3f}'.format(v_mean))
    print('    中央値 = {0:.3f}'.format(v_med))
    print('    Q1 = {0:.3f}    Q3 = {1:.3f}'.format(v_25p, v_75p))
    print('    95% CI = [{0:.3f}, {1:.3f}]'.format(v_025p, v_975p))
    txtout.write('    平均値 = {0:.3f}\n'.format(v_mean))
    txtout.write('    中央値 = {0:.3f}\n'.format(v_med))
    txtout.write('    Q1 = {0:.3f}    Q3 = {1:.3f}\n'.format(v_25p, v_75p))
    txtout.write('    95% CI = [{0:.3f}, {1:.3f}]\n'.format(v_025p, v_975p))

plt.title('主効果：{}'.format(NameB), fontsize = 20)
for k in range(b):
    sb.kdeplot(f_bs[:, k], label = LevelsB[k])
plt.yticks([])
plt.savefig(foutnm[:-4] + '_b.png')
SavedFiles.append(foutnm[:-4] + '_b.png')
plt.show()

print('\n交互作用：')
txtout.write('\n交互作用：\n')
for j in range(a):
    for k in range(b):
        print('\n{0}・{1}：'.format(LevelsA[j], LevelsB[k]))
        txtout.write('\n{0}・{1}：\n'.format(LevelsA[j], LevelsB[k]))
        v_mean = f_abs[:, j, k].mean()
        v_025p, v_25p, v_med, v_75p, v_975p = \
            np.percentile(f_abs[:, j, k], [2.5, 25, 50, 75, 97.5])
        print('    平均値 = {0:.3f}'.format(v_mean))
        print('    中央値 = {0:.3f}'.format(v_med))
        print('    Q1 = {0:.3f}    Q3 = {1:.3f}'.format(v_25p, v_75p))
        print('    95% CI = [{0:.3f}, {1:.3f}]'.format(v_025p, v_975p))
        txtout.write('    平均値 = {0:.3f}\n'.format(v_mean))
        txtout.write('    中央値 = {0:.3f}\n'.format(v_med))
        txtout.write('    Q1 = {0:.3f}    Q3 = {1:.3f}\n'.format(v_25p, v_75p))
        txtout.write('    95% CI = [{0:.3f}, {1:.3f}]\n'.format(v_025p, v_975p))

plt.title('交互作用：{0}・{1}'.format(NameA, NameB), fontsize = 20)
for j in range(a):
    for k in range(b):
        sb.kdeplot(f_abs[:, j, k], label = LevelsA[j] + '*' + LevelsB[k])
plt.yticks([])
plt.savefig(foutnm[:-4] + '_ab.png')
SavedFiles.append(foutnm[:-4] + '_ab.png')
plt.show()

es_a = []
es_b = []
es_ab = []
for a_s, b_s, ab_s, sgm_s in zip(f_as, f_bs, f_abs, fit['sgm']):
    es_a.append(np.std(a_s) / sgm_s)
    es_b.append(np.std(b_s) / sgm_s)
    es_ab.append(np.std(ab_s) / sgm_s)

es_a = np.array(es_a)
es_b = np.array(es_b)
es_ab = np.array(es_ab)

print('\n効果量（主効果）：{}'.format(NameA))
txtout.write('\n\n効果量（主効果）：{}\n'.format(NameA))
v_mean = es_a.mean()
v_025p, v_25p, v_med, v_75p, v_975p = \
    np.percentile(es_a, [2.5, 25, 50, 75, 97.5])
print('    平均値 = {0:.3f}'.format(v_mean))
print('    中央値 = {0:.3f}'.format(v_med))
print('    Q1 = {0:.3f}    Q3 = {1:.3f}'.format(v_25p, v_75p))
print('    95% CI = [{0:.3f}, {1:.3f}]'.format(v_025p, v_975p))
txtout.write('    平均値 = {0:.3f}\n'.format(v_mean))
txtout.write('    中央値 = {0:.3f}\n'.format(v_med))
txtout.write('    Q1 = {0:.3f}    Q3 = {1:.3f}\n'.format(v_25p, v_75p))
txtout.write('    95% CI = [{0:.3f}, {1:.3f}]\n'.format(v_025p, v_975p))

print('\n効果量（主効果）：{}'.format(NameB))
txtout.write('\n効果量（主効果）：{}\n'.format(NameB))
v_mean = es_b.mean()
v_025p, v_25p, v_med, v_75p, v_975p = \
    np.percentile(es_b, [2.5, 25, 50, 75, 97.5])
print('    平均値 = {0:.3f}'.format(v_mean))
print('    中央値 = {0:.3f}'.format(v_med))
print('    Q1 = {0:.3f}    Q3 = {1:.3f}'.format(v_25p, v_75p))
print('    95% CI = [{0:.3f}, {1:.3f}]'.format(v_025p, v_975p))
txtout.write('    平均値 = {0:.3f}\n'.format(v_mean))
txtout.write('    中央値 = {0:.3f}\n'.format(v_med))
txtout.write('    Q1 = {0:.3f}    Q3 = {1:.3f}\n'.format(v_25p, v_75p))
txtout.write('    95% CI = [{0:.3f}, {1:.3f}]\n'.format(v_025p, v_975p))

print('\n効果量（交互作用）：{0}・{1}'.format(NameA, NameB))
txtout.write('\n効果量（交互作用）：{0}・{1}\n'.format(NameA, NameB))
v_mean = es_ab.mean()
v_025p, v_25p, v_med, v_75p, v_975p = \
    np.percentile(es_ab, [2.5, 25, 50, 75, 97.5])
print('    平均値 = {0:.3f}'.format(v_mean))
print('    中央値 = {0:.3f}'.format(v_med))
print('    Q1 = {0:.3f}    Q3 = {1:.3f}'.format(v_25p, v_75p))
print('    95% CI = [{0:.3f}, {1:.3f}]'.format(v_025p, v_975p))
txtout.write('    平均値 = {0:.3f}\n'.format(v_mean))
txtout.write('    中央値 = {0:.3f}\n'.format(v_med))
txtout.write('    Q1 = {0:.3f}    Q3 = {1:.3f}\n'.format(v_25p, v_75p))
txtout.write('    95% CI = [{0:.3f}, {1:.3f}]\n'.format(v_025p, v_975p))

plt.title('効果量', fontsize = 20)
sb.kdeplot(es_a, label = '主効果：{}'.format(NameA))
sb.kdeplot(es_b, label = '主効果：{}'.format(NameB))
sb.kdeplot(es_ab, label = '交互作用')
plt.yticks([])
plt.savefig(foutnm[:-4] + '_es.png')
SavedFiles.append(foutnm[:-4] + '_es.png')
plt.show()

print("\n\nGjk's =")
txtout.write("\n\nGjk's =\n")
for j in range(a):
    for k in range(b):
        print('\nGjk({0},{1}):'.format(LevelsA[j], LevelsB[k]))
        txtout.write('\nGjk({0},{1}):\n'.format(LevelsA[j], LevelsB[k]))
        v_mean = fit['G'][:, j, k].mean()
        v_025p, v_25p, v_med, v_75p, v_975p = \
            np.percentile(fit['G'][:, j, k], [2.5, 25, 50, 75, 97.5])
        print('    平均値 = {0:.3f}'.format(v_mean))
        print('    中央値 = {0:.3f}'.format(v_med))
        print('    Q1 = {0:.3f}    Q3 = {1:.3f}'.format(v_25p, v_75p))
        print('    95% CI = [{0:.3f}, {1:.3f}]'.format(v_025p, v_975p))
        txtout.write('    平均値 = {0:.3f}\n'.format(v_mean))
        txtout.write('    中央値 = {0:.3f}\n'.format(v_med))
        txtout.write('    Q1 = {0:.3f}    Q3 = {1:.3f}\n'.format(v_25p, v_75p))
        txtout.write('    95% CI = [{0:.3f}, {1:.3f}]\n'.format(v_025p, v_975p))
   
txtout.close()
SavedFiles.append(txtoutnm)

print('\n\n保存ファイル:')
for nm in SavedFiles:
    print('    ', nm)
print()
